@@ -142,7 +142,6 @@ class TestFilesIndexer(tests.TestCase, TestFilesDBTable):
 		self.assertFilesDBConsistent(db)
 		self.assertFilesDBEquals(db, self.FILES)
 
-
 	def create_files(self, files):
 		for name in files:
 			if is_dir(name):
@@ -158,6 +157,42 @@ class TestFilesIndexer(tests.TestCase, TestFilesDBTable):
 				self.root.child(name).remove()
 
 
+class TestFilesIndexerWithCaseInsensitiveFilesytem(tests.TestCase, TestFilesDBTable):
+
+	def runTest(self):
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_MOCK)
+		folder._fs.set_case_sensitive(False)
+
+		db = sqlite3.connect(':memory:')
+		db.row_factory = sqlite3.Row
+		indexer = FilesIndexer(db, folder)
+
+		def check_and_update_all():
+			checker = FilesIndexChecker(indexer.db, indexer.folder)
+			checker.queue_check()
+			for out_of_date in checker.check_iter():
+				if out_of_date:
+					for i in indexer.update_iter():
+						pass
+			indexer.db.commit()
+
+		for name in ('aaa.txt', 'bbb.txt', 'ccc.txt'):
+			folder.file(name).write('Test 123\n')
+
+		check_and_update_all()
+		self.assertFilesDBConsistent(db)
+		self.assertFilesDBEquals(db, ('aaa.txt', 'bbb.txt', 'ccc.txt'))
+
+		mtime = folder.mtime()
+		folder.file('aaa.txt').moveto(folder.file('AAA.txt'))
+		self.assertEqual(list(folder.list_names()), ['AAA.txt', 'bbb.txt', 'ccc.txt'])
+		self.assertNotEqual(folder.mtime(), mtime)
+
+		check_and_update_all()
+		self.assertFilesDBConsistent(db)
+		self.assertFilesDBEquals(db, ('AAA.txt', 'bbb.txt', 'ccc.txt'))
+
+
 class TestPagesIndexer(TestPagesDBTable, tests.TestCase):
 
 	FILES = tuple(map(os_native_path, (
@@ -225,6 +260,8 @@ class TestPagesIndexer(TestPagesDBTable, tests.TestCase):
 		def cb_filter_func(name, o, a):
 			if name == 'page-changed':
 				row, content = a
+			elif name == 'page-row-changed':
+				row, oldrow = a
 			else:
 				row, = a
 
@@ -288,7 +325,8 @@ class TestPagesIndexer(TestPagesDBTable, tests.TestCase):
 		self.assertEqual(set(signals['page-row-changed']), set(['foo']))
 						 # "foo" has source that is deleted before children
 		self.assertEqual(set(signals['page-row-deleted']), set(self.PAGES))
-		self.assertEqual(signals['page-changed'], [])
+		self.assertEqual(signals['page-changed'], ['foo'])
+						 # "foo" has source that is deleted before children
 
 
 from zim.utils import natural_sort_key
@@ -315,7 +353,7 @@ class TestLinksIndexer(tests.TestCase):
 		pi = PagesIndexer(db, None, tests.MockObject())
 		for i, name, cont in self.PAGES:
 			db.execute(
-				'INSERT INTO pages(id, name, sortkey, parent) VALUES (?, ?, ?, 1)',
+				'INSERT INTO pages(id, name, sortkey, parent, source_file) VALUES (?, ?, ?, 1, 1)',
 				(i, name, natural_sort_key(name))
 			)
 
@@ -348,7 +386,7 @@ class TestLinksIndexer(tests.TestCase):
 			(r['source'], r['target'])
 				for r in db.execute('SELECT * FROM links')
 		)
-		self.assertEqual(links, [(3,2), (3,4)])
+		self.assertEqual(links, [(3, 2), (3, 4)])
 
 		###
 		pageindexer.setObjectAccess('remove_page')
