@@ -12,11 +12,13 @@ straight forward API.
 import gtk
 import logging
 
-from zim.fs import File, Dir
+from zim.fs import File, Dir, FS
+from zim.newfs import LocalFolder
 from zim.notebook import Path
 from zim.parsing import is_url_re, url_encode, link_type, URL_ENCODE_READABLE
-from zim.formats import get_format, ParseTree, TreeBuilder
-from zim.exporter import StaticLinker
+from zim.formats import get_format, ParseTree, ParseTreeBuilder, \
+	FORMATTEDTEXT, IMAGE, LINK
+from zim.export.linker import StaticExportLinker
 
 
 logger = logging.getLogger('zim.gui.clipboard')
@@ -72,13 +74,14 @@ TEXT_TARGET_NAMES = tuple([target[0] for target in TEXT_TARGETS])
 
 # All targets that we can convert to a parsetree, in order of choice
 PARSETREE_ACCEPT_TARGETS = (
-	PARSETREE_TARGET,
-	INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET,
+        PARSETREE_TARGET,
+        INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET,
 ) + IMAGE_TARGETS + URI_TARGETS + TEXT_TARGETS
 PARSETREE_ACCEPT_TARGET_NAMES = tuple([target[0] for target in PARSETREE_ACCEPT_TARGETS])
 #~ print 'ACCEPT', PARSETREE_ACCEPT_TARGET_NAMES
 
 
+
 # Mimetype text/uri-list is used for drag n drop of URLs
 # it is plain text encoded list of urls, separated by \r\n
 # Since not all apps follow the standard exactly, do allow for
@@ -102,10 +105,79 @@ def unpack_urilist(text):
 	# FIXME be tolerant here for file://path/to/file uris here
 	text = text.strip('\x00') # Found trailing NULL character on windows
 	lines = text.splitlines() # takes care of \r\n
-	return [ line.decode('utf-8')
-		for line in lines if line and not line.isspace() ]
+	return [line.decode('utf-8')
+		for line in lines if line and not line.isspace()]
 		# Just to be sure we also skip empty or whitespace lines
 
+# TODO: Probably the serialize formats can replace custom copy/paste
+# handlers in TextView and TextBuffer as well
+
+
+def textbuffer_register_serialize_formats(buffer, notebook, page):
+	if gtk.pygtk_version >= (2, 10):
+		buffer.register_serialize_format('text/x-zim-parsetree', serialize_parse_tree)
+		buffer.register_deserialize_format('text/x-zim-parsetree', deserialize_parse_tree, (notebook, page))
+		for name in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) + URI_TARGET_NAMES:
+			buffer.register_deserialize_format(name, deserialize_urilist, (notebook, page))
+		for name in IMAGE_TARGET_NAMES: # FIXME, should we limit the list ?
+			buffer.register_deserialize_format(name, deserialize_image, (name, notebook, page))
+
+def serialize_parse_tree(register_buf, content_buf, start, end):
+	tree = content_buf.get_parsetree((start, end))
+	xml = tree.tostring().encode('utf-8')
+	return xml
+
+def deserialize_parse_tree(register_buf, content_buf, iter, data, create_tags, user_data):
+	notebook, path = user_data
+	tree = ParseTree().fromstring(data)
+	tree.resolve_images(notebook, path)
+	content_buf.insert_parsetree(iter, tree, interactive=True)
+	return True
+
+def deserialize_urilist(register_buf, content_buf, iter, data, create_tags, user_data):
+	notebook, path = user_data
+	links = unpack_urilist(data)
+	tree = _link_tree(links, notebook, path)
+	content_buf.insert_parsetree(iter, tree, interactive=True)
+	return True
+
+def deserialize_image(register_buf, content_buf, iter, data, create_tags, user_data):
+	# Implementation note: we follow gtk_selection_get_pixbuf() in usage of
+	# gtk.PixbufLoader to capture clipboard data in a pixbuf object.
+	# We could skip this, but it allows for on-the-fly conversion of the data
+	# type.
+	mimetype, notebook, path = user_data
+
+	# capture image
+	loader = gtk.gdk.PixbufLoader()
+	loader.write(data)
+	loader.close()
+	pixbuf = loader.get_pixbuf()
+
+	# save it as an attachment
+	dir = notebook.get_attachments_dir(path)
+	if not dir.exists():
+		logger.debug("Creating attachment dir: %s", dir)
+		dir.touch()
+
+	format, extension = _get_image_info(mimetype)
+	if format is None or format == 'bmp':
+		# default to png format
+		# special casing bmp since many window apps use it internally
+		# but is quite large to store, so compress by using png
+		format, extension = 'png', 'png'
+
+	file = dir.new_file('pasted_image.%s' % extension)
+	logger.debug("Saving image from clipboard to %s", file)
+	pixbuf.save(file.path, format)
+	FS.emit('path-created', file) # notify version control
+
+	# and insert it in the page
+	links = [file.uri]
+	tree = _link_tree(links, notebook, path)
+	content_buf.insert_parsetree(iter, tree, interactive=True)
+	return True
+
 
 def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 	'''Function to get a parsetree based on the selectiondata contents
@@ -141,7 +213,7 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 		# try to catch this situation by a check here
 		text = selectiondata.get_text()
 		if text:
-			return get_format('plain').Parser().parse(text.decode('utf-8'))
+			return get_format('plain').Parser().parse(text.decode('utf-8'), partial=True)
 		else:
 			return None
 	elif targetname in IMAGE_TARGET_NAMES:
@@ -151,6 +223,8 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 			return None
 
 		dir = notebook.get_attachments_dir(path)
+		assert isinstance(dir, LocalFolder) or hasattr(dir, '_folder') and isinstance(dir._folder, LocalFolder)
+			# XXX: assert we have local path  - HACK to deal with FilesAttachmentFolder
 		if not dir.exists():
 			logger.debug("Creating attachment dir: %s", dir)
 			dir.touch()
@@ -165,6 +239,7 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 		file = dir.new_file('pasted_image.%s' % extension)
 		logger.debug("Saving image from clipboard to %s", file)
 		pixbuf.save(file.path, format)
+		FS.emit('path-created', file) # notify version control
 
 		links = [file.uri]
 		return _link_tree(links, notebook, path)
@@ -176,11 +251,11 @@ def _link_tree(links, notebook, path):
 	# Convert a list of links (of any type) into a parsetree
 	#~ print 'LINKS: ', links
 	#~ print 'NOTEBOOK and PATH:', notebook, path
-	builder = TreeBuilder()
-	builder.start('zim-tree')
+	builder = ParseTreeBuilder()
+	builder.start(FORMATTEDTEXT)
 	for i in range(len(links)):
 		if i > 0:
-			builder.data(' ')
+			builder.text(' ')
 
 		link = links[i]
 		type = link_type(link)
@@ -196,27 +271,23 @@ def _link_tree(links, notebook, path):
 
 		if isimage:
 			src = notebook.relative_filepath(file, path) or file.uri
-			builder.start('img', {'src': src})
-			builder.end('img')
+			builder.append(IMAGE, {'src': src})
 		elif link.startswith('@'):
 			# FIXME - is this ever used ??
-			builder.start('tag', {'name': links[i][1:]})
-			builder.data(links[i])
-			builder.end('tag')
+			builder.append(TAG, {'name': links[i][1:]}, links[i])
 		else:
 			if type == 'page':
-				href = Path(notebook.cleanup_pathname(link)) # Assume links are always absolute
-				link = notebook.relative_link(path, href) or link
+				target = Path(Path.makeValidPageName(link)) # Assume links are always absolute
+				href = notebook.pages.create_link(path, target)
+				link = href.to_wiki_link()
 			elif type == 'file':
 				file = File(link) # Assume links are always URIs
 				link = notebook.relative_filepath(file, path) or file.uri
 
-			builder.start('link', {'href': link})
-			builder.data(link)
-			builder.end('link')
+			builder.append(LINK, {'href': link}, link)
 
-	builder.end('zim-tree')
-	tree = ParseTree(builder.close())
+	builder.end(FORMATTEDTEXT)
+	tree = builder.get_parsetree()
 	tree.resolve_images(notebook, path)
 	tree.decode_urls()
 	return tree
@@ -288,6 +359,25 @@ class UriItem(ClipboardItem):
 				selectiondata.set_text(self.uri)
 
 
+class InterWikiLinkItem(UriItem):
+	'''Like L{UriItem} but with special case for zim parsetree'''
+
+	targets = (PARSETREE_TARGET,) + UriItem.targets
+
+	def __init__(self, href, url):
+		UriItem.__init__(self, url)
+		self.interwiki_href = href
+
+	def _get(self, clipboard, selectiondata, id, *a):
+		logger.debug("Clipboard requests data as '%s', we have an interwiki link", selectiondata.target)
+		if id == PARSETREE_TARGET_ID:
+			tree = _link_tree((self.interwiki_href,), None, None)
+			xml = tree.tostring().encode('utf-8')
+			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
+		else:
+			UriItem._get(self, clipboard, selectiondata, id, *a)
+
+
 class ParseTreeItem(ClipboardItem):
 	'''Clipboard wrapper for a L{ParseTree}.'''
 
@@ -316,46 +406,23 @@ class ParseTreeItem(ClipboardItem):
 			xml = self.parsetree.tostring().encode('utf-8')
 			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
 		elif id == HTML_TARGET_ID:
-			# FIXME - HACK - dump and parse as wiki first to work
-			# around glitches in pageview parsetree dumper
-			# main visibility when copy pasting bullet lists
-			# Same hack in print to browser plugin
-			dumper = get_format('wiki').Dumper()
-			text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
-			parser = get_format('wiki').Parser()
-			parsetree = parser.parse(text)
-			#--
 			dumper = get_format('html').Dumper(
-				linker=StaticLinker('html', self.notebook, self.path) )
-			html = ''.join( dumper.dump(parsetree) )
+				linker=StaticExportLinker(self.notebook, source=self.path))
+			html = ''.join(dumper.dump(self.parsetree))
 			html = wrap_html(html, target=selectiondata.target)
 			#~ print 'PASTING: >>>%s<<<' % html
 			selectiondata.set(selectiondata.target, 8, html)
 		elif id == TEXT_TARGET_ID:
 			logger.debug("Clipboard requested text, we provide '%s'" % self.format)
-			if self.format != 'wiki':
-				# FIXME - HACK - dump and parse as wiki first to work
-				# around glitches in pageview parsetree dumper
-				# main visibility when copy pasting bullet lists
-				# Same hack in print to browser plugin
-				dumper = get_format('wiki').Dumper()
-				text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
-				parser = get_format('wiki').Parser()
-				parsetree = parser.parse(text)
-				if self.parsetree.ispartial:
-					parsetree.getroot().attrib['partial'] = True
-				#--
-			else:
-				parsetree = self.parsetree
 			#~ print ">>>>", self.format, parsetree.tostring()
 
 			if self.format in ('wiki', 'plain'):
 				dumper = get_format(self.format).Dumper()
 			else:
 				dumper = get_format(self.format).Dumper(
-					linker=StaticLinker(self.format, self.notebook, self.path) )
+					linker=StaticExportLinker(self.notebook, source=self.path))
 
-			text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
+			text = ''.join(dumper.dump(self.parsetree)).encode('utf-8')
 			selectiondata.set_text(text)
 		else:
 			assert False, 'Unknown target id %i' % id
@@ -509,6 +576,15 @@ class ClipboardManager(object):
 		item = PageLinkItem(notebook, path)
 		self.set(item)
 
+	def set_interwikilink(self, href, url):
+		'''Copy an interwiki link to the clipboard
+		@param href: the link as shown in zim, e.g. "wp?foobar"
+		@param url: the expanded url for this interwiki link, e.g.
+		"http://en.wikipedia.org/wiki/foobar"
+		'''
+		item = InterWikiLinkItem(href, url)
+		self.set(item)
+
 	def set_uri(self, uri):
 		'''Copy an uri to the clipboard
 		@param uri: an uri as string, or an object with an attribute C{uri}
@@ -613,9 +689,9 @@ class Win32HtmlFormat:
 		lenPrefix = len(dummyPrefix)
 
 		prefix = klass.MARKER_BLOCK_OUTPUT % (
-			lenPrefix, len(html)+lenPrefix,
-			fragmentStart+lenPrefix, fragmentEnd+lenPrefix,
-			selectionStart+lenPrefix, selectionEnd+lenPrefix,
+			lenPrefix, len(html) + lenPrefix,
+			fragmentStart + lenPrefix, fragmentEnd + lenPrefix,
+			selectionStart + lenPrefix, selectionEnd + lenPrefix,
 			source
 		)
 		return prefix + html
