@@ -8,9 +8,9 @@ from zim.gui.widgets import Dialog
 class PropertiesDialog(Dialog):
 
 	def __init__(self, ui):
-		Dialog.__init__(self, ui, _('Properties')) # T: Dialog title
+		Dialog.__init__(self, ui, _('Properties'), help='Help:Properties') # T: Dialog title
 		label = gtk.Label()
-		label.set_markup('<b>'+_('Notebook Properties')+'</b>')
+		label.set_markup('<b>' + _('Notebook Properties') + '</b>')
 			# T: Section in notebook dialog
 		label.set_alignment(0.0, 0.5)
 		self.vbox.pack_start(label, False)
@@ -29,3 +29,5 @@ class PropertiesDialog(Dialog):
 			properties = self.form.copy()
 			self.ui.notebook.save_properties(**properties)
 		return True
+
+## TODO: put a number of properties in an expander with a lable "Advanced"
